==> fotoalbum-0.33.tar.bz2: fotoalbum-0.33/src/shared/filetools.h <==
/*
 * Copyright (C) 2005-2007, 2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef FILETOOLS_H
# define FILETOOLS_H

# include <sys/types.h> 
# include <string>
# include <fstream>
# include "filetools_cfg.h"

#ifdef WIN32_FLTK13
# include <ext/stdio_filebuf.h>
#endif


class FileTools
{
public:
  
  /**
   * Creates a relative path.
   * refPath should be a directory including the trailing slash.
   */
  static std::string createRelPath(std::string refPath,
                                   std::string absPath, 
                                   bool        fLead = true);
  
  /** 
   * Creates an absolute path from rel and base. Can also be used to clean up paths containing "../" and "./"
   * If rel begins with a "/", it will be returned unchanged.
   */
  static std::string createAbsPath(std::string base,
                                   std::string rel);
  
  /**
   * Does the file exist?
   */
  static bool fileExists(std::string path);
  
  /**
   * Öffnet eine Datei zum Schreiben. Wenn es die Datei vorher schon gab,
   * wird sie unter name_backup gespeichert. name_backup wird dabei 
   * überschrieben, wenn es schon vorhanden ist.
   */
  static void openAndCreateBackup(std::ofstream& out, std::string path);
  
  /**
   * Compares ref to result.
   * 
   * 1:  ok, result is newer than ref (or the same age)
   * -1: there's a file, but it is too old
   * -2: there isn't even a file...
   */
  static int isFileUpToDate(std::string ref, std::string result);
  
  /**
   * returns 0 if everything is fine
   */
  static int makeSureDirExists(std::string dirname);
  
  /**
   * Versucht $HOME/.appdirname (linux) oder
   * $USERPROFILE/appdirname zu öffnen oder anzulegen.
   * Wenn das klappt, wird dieses Verzeichnis zurückgegeben,
   * ansonsten das aktuelle Verzeichnis.
   * Das zurückgegebene Verzeichnis endet mit einem "/".
   */
  static std::string getHomeDir(std::string appdirname);

  /**
   * Wohin zeigt ein Link?
   */
  static std::string getLinkTarget(std::string path);

  /**
   * Delete file.
   * Returns nonzero on error.
   */
  static int rm(std::string filename);
  
  /**
   * Get size of file (in bytes). Returns -1 on error.
   */
  static int getSize(std::string filename);
  
  /**
   * Returns name and suffix of a file, strips leading directory information.
   */
  static std::string name(std::string pathAndName);
  
  /**
   * Returns path (with trailing '/', unless there is no path) of a file, 
   * strips name and suffix.
   */
  static std::string path(std::string pathAndName);
  
  /**
   * Removes everything after the first dot.
   */
  static std::string removeSuffix(std::string name);
  
  /**
   * Returns everything starting at the last dot.
   */
  static std::string getSuffix(std::string name);
  
  /**
   * Returns a string without 'difficult' characters.
   * Only numbers, a-z, underscore are allowed.
   * One may add characters which are allowed on any filesystem
   * without quoting.
   * '/' and '\' must not be included.    
   */
  static std::string cleanString(std::string name, std::string allowed = "");
  
  /**
   * Copies a file. 
   * dest can be a file or a directory (ends with '/').
   */
  static int copy(std::string dest, std::string src);
  
  /**
   * Moves a file.
   * dest can be a file or a directory (ends with '/').
   */
  static int move(std::string dest, std::string src);
  
  /**
   * Results in a path with trailing '/', but no leading things.
   * It is used to concat paths.
   */
  static std::string stripAbsFromPath(std::string in);
  
  static bool isDir(std::string name);
  
  static bool isLink(std::string name);
  
  static bool isWriteable(std::string name);
  
  static std::string getWorkingDir();
  
  /**
   * The link path is src, it points to dest.
   */
  static int symlink(std::string dest, std::string src);

  static time_t Get_mtime(std::string path);

  static std::string BackslashToSlash(std::string in);
  
  static int stat(std::string strName, struct stat* attribs);
};


/**
 * A class to create a temporary text file and write text to it. 
 * It returns the filename to enable reading it.
 * This is far from perfect at least on one platform.
 * 
 * The main reason to write it was to achieve some portability and
 * a mostly stable interface.
 */
class TemporaryTextFile
{
public:

  static std::string GetGlobalTempDirPath();
  
   /**
    * returns a file descriptor
    */
  static int GetTempFD(std::string& path);
  
  bool isOK() { return(fOK); };
  
  std::string getPathAndName() { return(FilenameAndPath); };
  
# if (TEMP_TEXTFILE_VARIANT == 0)
  int filedescr;
  
  /**
   * Uses mkstemp. "man mkstemp" advises to use "man 3 tmpfile". But how to 
   * get the filename in this case?
   */
  TemporaryTextFile(std::string name_template);
# endif
  
# if (TEMP_TEXTFILE_VARIANT == 1)
  std::ofstream outfile;
  
  /**
   * Poor man's method
   */
  TemporaryTextFile(std::string name_template);
# endif
  
# if (TEMP_TEXTFILE_VARIANT == 2)
  FILE*        tmpfiledescr;
  
  /**
   * using "man 3 tmpfile"
   */
  TemporaryTextFile();
# endif
  
  /**
   * Write some characters to the file
   */
  void write(std::string line);
  
  /**
   * Close the file, but don't delete it!
   */
  void close();
  
private:
  
  std::string FilenameAndPath;
  
  bool fOK;
};


#endif

==> fotoalbum-0.201.tar.bz2: fotoalbum-0.201/src/shared/filetools.h <==
/*
 * Copyright (C) 2005-2007, 2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef FILETOOLS_H
# define FILETOOLS_H

# include <sys/types.h> 
# include <string>
# include <fstream>
# include "filetools_cfg.h"

#ifdef WIN32_FLTK13
# include <ext/stdio_filebuf.h>
#endif


class FileTools
{
public:
  
  /**
   * Creates a relative path.
   * refPath should be a directory including the trailing slash.
   */
  static std::string createRelPath(std::string refPath,
                                   std::string absPath, 
                                   bool        fLead = true);
  
  /** 
   * Creates an absolute path from rel and base. Can also be used to clean up paths containing "../" and "./"
   * If rel begins with a "/", it will be returned unchanged.
   */
  static std::string createAbsPath(std::string base,
                                   std::string rel);
  
  /**
   * Does the file exist?
   */
  static bool fileExists(std::string path);
  
  /**
   * Öffnet eine Datei zum Schreiben. Wenn es die Datei vorher schon gab,
   * wird sie unter name_backup gespeichert. name_backup wird dabei 
   * überschrieben, wenn es schon vorhanden ist.
   */
  static void openAndCreateBackup(std::ofstream& out, std::string path);
  
  /**
   * Compares ref to result.
   * 
   * 1:  ok, result is newer than ref (or the same age)
   * -1: there's a file, but it is too old
   * -2: there isn't even a file...
   */
  static int isFileUpToDate(std::string ref, std::string result);
  
  /**
   * returns 0 if everything is fine, negative on error:
   * -1 other error
   * -200 read only filesystem
   */
  static int makeSureDirExists(std::string dirname);
  
  /**
   * Versucht $HOME/.appdirname (linux) oder
   * $USERPROFILE/appdirname zu öffnen oder anzulegen.
   * Wenn das klappt, wird dieses Verzeichnis zurückgegeben,
   * ansonsten das aktuelle Verzeichnis.
   * Das zurückgegebene Verzeichnis endet mit einem "/".
   */
  static std::string getHomeDir(std::string appdirname);

  /**
   * Wohin zeigt ein Link?
   */
  static std::string getLinkTarget(std::string path);

  /**
   * Delete file.
   * Returns nonzero on error.
   */
  static int rm(std::string filename);
  
  /**
   * Get size of file (in bytes). Returns -1 on error.
   */
  static int getSize(std::string filename);
  
  /**
   * Returns name and suffix of a file, strips leading directory information.
   */
  static std::string name(std::string pathAndName);
  
  /**
   * Returns path (with trailing '/', unless there is no path) of a file, 
   * strips name and suffix.
   */
  static std::string path(std::string pathAndName);
  
  /**
   * Removes everything after the first dot.
   */
  static std::string removeSuffix(std::string name);
  
  /**
   * Returns everything starting at the last dot.
   */
  static std::string getSuffix(std::string name);
  
  /**
   * Returns a string without 'difficult' characters.
   * Only numbers, a-z, underscore are allowed.
   * One may add characters which are allowed on any filesystem
   * without quoting.
   * '/' and '\' must not be included.    
   */
  static std::string cleanString(std::string name, std::string allowed = "");
  
  /**
   * Copies a file. 
   * dest can be a file or a directory (ends with '/').
   */
  static int copy(std::string dest, std::string src);
  
  /**
   * Moves a file.
   * dest can be a file or a directory (ends with '/').
   */
  static int move(std::string dest, std::string src);
  
  /**
   * Results in a path with trailing '/', but no leading things.
   * It is used to concat paths.
   */
  static std::string stripAbsFromPath(std::string in);
  
  static bool isDir(std::string name);
  
  static bool isLink(std::string name);
  
  static bool isWriteable(std::string name);
  
  static std::string getWorkingDir();
  
  /**
   * The link path is src, it points to dest.
   */
  static int symlink(std::string dest, std::string src);

  static time_t Get_mtime(std::string path);

  static std::string BackslashToSlash(std::string in);
  
  static int stat(std::string strName, struct stat* attribs);
};


/**
 * A class to create a temporary text file and write text to it. 
 * It returns the filename to enable reading it.
 * This is far from perfect at least on one platform.
 * 
 * The main reason to write it was to achieve some portability and
 * a mostly stable interface.
 */
class TemporaryTextFile
{
public:

  static std::string GetGlobalTempDirPath();
  
   /**
    * returns a file descriptor
    */
  static int GetTempFD(std::string& path);
  
  bool isOK() { return(fOK); };
  
  std::string getPathAndName() { return(FilenameAndPath); };
  
# if (TEMP_TEXTFILE_VARIANT == 0)
  int filedescr;
  
  /**
   * Uses mkstemp. "man mkstemp" advises to use "man 3 tmpfile". But how to 
   * get the filename in this case?
   */
  TemporaryTextFile(std::string name_template);
# endif
  
# if (TEMP_TEXTFILE_VARIANT == 1)
  std::ofstream outfile;
  
  /**
   * Poor man's method
   */
  TemporaryTextFile(std::string name_template);
# endif
  
# if (TEMP_TEXTFILE_VARIANT == 2)
  FILE*        tmpfiledescr;
  
  /**
   * using "man 3 tmpfile"
   */
  TemporaryTextFile();
# endif
  
  /**
   * Write some characters to the file
   */
  void write(std::string line);
  
  /**
   * Close the file, but don't delete it!
   */
  void close();
  
private:
  
  std::string FilenameAndPath;
  
  bool fOK;
};


#endif
